///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Flexporter SDK Beta v1.1
 *	\file		FlexporterSDK.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __FLEXPORTER_SDK__
#define __FLEXPORTER_SDK__

//! SDK Version number
#define FLEXPORTER_VERSION	101

//! Using ICE Core
#define DEFAULT_NEWDELETE
#include "IceCore.h"
using namespace IceCore;

// Carbon-copy from IceMeshBuilder

	//! Submesh properties.
	struct MESHBUILDERSUBMESH
	{
		sdword			MatID;								//!< MaterialID for this submesh
		udword			SmGrp;								//!< Smoothing groups for this submesh
		udword			NbFaces;							//!< Number of faces in this submesh
		udword			NbVerts;							//!< Number of vertices in this submesh
		udword			NbSubstrips;						//!< Number of strips in this submesh
	};

	//! Material properties.
	struct MESHBUILDERMATINFO
	{
		sdword			MatID;								//!< This material's ID
		udword			NbFaces;							//!< Number of faces having this material
		udword			NbVerts;							//!< Related number of exported vertices
		udword			NbSubmeshes;						//!< Number of related submeshes
	};

	//! The topology structure.
	struct MESHBUILDERTOPOLOGY
	{
		uword				NbFaces;						//!< Total number of faces
		uword				NbSubmeshes;					//!< Total number of submeshes (e.g. 6 for the reference cube)
		union {
			uword*			StripRuns;						//!< Obsolete
			uword*			VRefs;							//!< Vertex references (3 refs for each face)
		};
		union {
			uword*			StripLengths;					//!< Obsolete
			uword*			FacesInSubmesh;					//!< Number of faces in each submesh
		};
		float*				Normals;						//!< Face normals
		MESHBUILDERSUBMESH*	SubmeshProperties;				//!< NbSubmeshes structures
	};

	//! The geometry structure.
	struct MESHBUILDERGEOMETRY
	{
		// Basic data
		uword				NbGeomPts;						//!< Number of vertices in the original mesh
		uword				NbTVerts;						//!< Number of mapping coordinates in the original mesh
		uword				NbVerts;						//!< Number of vertices in the final mesh (some may have been duplicated) = sum of all NbVerts in MESHBUILDERSUBMESHes
		// Indices
		udword*				VertsRefs;						//!< Vertex indices (only !=null if mIndexedGeo, else vertices are duplicated). Index in Verts.
		udword*				TVertsRefs;						//!< UVW indices (only !=null if mIndexedUVW, else UVWs are duplicated). Index in TVerts.
		// Vertex data
		float*				Verts;							//!< List of vertices, may be duplicated or not
		float*				TVerts;							//!< List of UV(W) mappings, may be duplicated or not.
		float*				Normals;						//!< Vertex normals. Can't be indexed.
		float*				CVerts;							//!< List of vertex colors. Can't be indexed.
		udword				NormalInfoSize;					//!< Size of the NormalInfo field.
		udword*				NormalInfo;						//!< Information used to rebuild normals in realtime. See below.
	};

	// More about NormalInfo:
	//
	// NormalInfo contains some extra information used to rebuild vertex-normals in realtime, by averaging
	// a number of face-normals. Each vertex-normal depends on a various number of face-normals. The exact
	// number depends on the mesh topology, but also depends on the smoothing groups.
	//
	// NormalInfo contains data to rebuild one normal/vertex, ie to rebuild NbVerts normals.
	// Format is, for each vertex:
	//		udword		Count				a number of faces
	//		udword		Ref0, Ref1,...		a list of Count face indices
	//
	// To rebuild vertex-normals in realtime you need to:
	// 1) Rebuild all face-normals (which is easy)
	// 2) For each vertex, add Count face-normals according to NormalInfo, then normalize the summed vector.
	//
	// Other techniques exist, of course.

	//! The materials structure.
	struct MESHBUILDERMATERIALS
	{
		udword				NbMtls;							//!< Number of materials found.
		MESHBUILDERMATINFO*	MaterialInfo;					//!< One structure for each material.
	};

	//! Result structure.
	struct MESHBUILDERRESULT
	{
		//! Constructor
		MESHBUILDERRESULT()
		{
			WireframeData	= null;
			ProgressiveMesh	= null;
		}
		void*					WireframeData;				//!< Obsolete.
		MESHBUILDERTOPOLOGY		Topology;					//!< Result topology.
		MESHBUILDERGEOMETRY		Geometry;					//!< Result geometry
		MESHBUILDERMATERIALS	Materials;					//!< Result materials
		udword*					ProgressiveMesh;			//!< Obsolete.
	};

	class MeshBuilder;

// ~Carbon-copy from IceMeshBuilder

//! Include Flexporter headers
#include "IceAddPRS.h"
#include "IceTimeInfo.h"
#include "IceExportSettings.h"
#include "IceBaseInfo.h"
#include "IceAddCamera.h"
#include "IceAddLight.h"
#include "IceAddMaterial.h"
#include "IceAddTexture.h"
#include "IceAddMesh.h"
#include "IceAddController.h"
#include "IceAddHelper.h"
#include "IceAddMotion.h"
#include "IceAddShape.h"
#include "IceBaseFormat.h"
#include "IceCharacterStudioNodes.h"

#define DLLEXPORT __declspec(dllexport)

// Exported functions

//! A character string describing the exporter. Appears in Flexporter's Options panel.
DLLEXPORT const char*		ExporterDescription();

//! The extension used for the plug-in's format. (e.g. "txt" for default ascii exporter)
DLLEXPORT const char*		FormatExtension();

//! Returns a pointer to the unique instance of an exporter.
DLLEXPORT ExportFormat*		GetExporter();

//! Returns the default settings for the exporter.
DLLEXPORT ExportSettings*	GetDefaultSettings();

//! Returns FLEXPORTER_VERSION.
DLLEXPORT int				Version();

#endif // __FLEXPORTER_SDK__
