///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Main file for IceCore.dll.
 *	\file		IceCore.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICECORE_H__
#define __ICECORE_H__

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Compilation messages

#ifndef __cplusplus
	#error ICE needs C++ compilation...
#endif

#if defined(ICECORE_EXPORTS)
	#pragma message("Compiling ICE Core")
#elif !defined(ICECORE_EXPORTS)
	#pragma message("Using ICE Core")
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes

// Windows includes
#ifdef WIN32
#include <windows.h>												// General purpose Win32 functions
#include <windowsx.h>
#endif // WIN32

// DOS includes
#ifdef MSDOS
#include <dos.h>
#define DIR_STRUCT struct find_t
#define FIND_FIRST( n, d, a ) _dos_findfirst( n, a, d )
#define FIND_NEXT _dos_findnext
#define DIR_FILE_NAME name
#endif // MSDOS

// VMS includes
#ifdef VMS
#include <ssdef.h>
#include <rmsdef.h>
#include <descrip.h>
#endif // VMS

// Standard includes
#ifdef _DEBUG
#include <crtdbg.h>													// C runtime debug functions
#endif
#include <stdio.h>													// Standard Input/Output functions
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>													// Assertion functions
#include <shlobj.h>
#include <mmsystem.h>
#include <stdarg.h>
#include <time.h>
#include <float.h>

// ICE includes
	#include "IcePreprocessor.h"
	#include "IceTypes.h"
	#include "IceCPUID.h"
	#include "IceFPU.h"
	#include "IceMemoryManager.h"
	#include "IceDll.h"
	#include "IceBuffer.h"
		#include "IceQSort.h"

	namespace IceCore
	{
		#include "IceInstrumentTrace.h"
		#include "IceString.h"
		#include "IceError.h"
		#include "IceFile.h"

		#include "IceContainer.h"
		#include "IceCustomArray.h"
		#include "IceDynamicList.h"

//		#include "IceQSort.h"
		#include "IceRevisitedRadix.h"

		#include "IceProfiler.h"
		#include "IceImportExport.h"
		#include "IceRefTracker.h"
		#include "IceCell.h"
		#include "IceKernel.h"

		#include "IceRegistry.h"
	}

	// Log message
	ICECORE_API void Log(ubyte value);
	ICECORE_API void Log(uword value);
	ICECORE_API void Log(udword value);
	ICECORE_API void Log(float value);
	ICECORE_API void Log(sbyte value);
	ICECORE_API void Log(sword value);
	ICECORE_API void Log(sdword value);
	ICECORE_API void Log(const char* msg);
	ICECORE_API	void Log(LPSTR fmt, ...);

	ICECORE_API bool CheckSingleRun(const char* appname);
	ICECORE_API void Dump(const char* filename, const char* text);

	//! A function to test the DLL components.
	ICECORE_API void TestIceCore();

#endif // __ICECORE_H__













/*


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Definition of the Irion's standard functions' namespace
// Some functions are hidded for the user namespace

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Includes
	#include "IrionString.h"

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// IrionBasics Namespace definition
	namespace IrionBasics
	{

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// MACROS
		#define	FINDFILE_DIR			0x01
		#define	FINDFILE_HIDDEN			0x02
		#define	FINDFILE_NORMAL			0x04
		#define	FINDFILE_READONLY		0x08

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Prototypes
		class File;

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//
		//                       Class FindFiles Definition
		//
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		class CLASS_EXP FindFiles
		{
		public:
						FindFiles	(ctxt filepathname);
				bool	FindNext	();
				bool	IsValid		()								{ return (mH==(void*)-1) ? false : true; }

		public:
			String		mFile;
			udword		mAttrbs;
			udword		mSize;

		protected:
			WIN32_FIND_DATA		mFindData;
			HANDLE				mH;
		};

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Timer management functions
		typedef		void (__stdcall TIMERCALLBACK)	(udword tid, udword user);
		IRIONAPI	void			TimerGetCaps	(float& minres, float& maxres);												// Retrieve the caps of the timer device
		IRIONAPI	udword			TimerSetEvent	(float delay, bool oneshot, TIMERCALLBACK* cb, udword user);				// Set a timer event
		IRIONAPI	bool			TimerKillEvent	(udword tid);																// Kill a timer event

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Profiling functions
		#define		rdtsc __asm _emit 0x0F __asm _emit 0x31

#if defined(_IRION_BASIC)
					void			InitProfiling	();
					void __stdcall	ProfTimerCB		(udword tid, udword user);
					void			InitTimerMgr	();
					void			RestoreTimerMgr	();
#endif																								// defined(_IRION_BASIC)

#if defined(_IRION_BASIC) && !defined(_IRIONBASICS_CPP)
		extern		udword			CPUFreq;
		extern		udword			CPUFreqOk;
#elif !defined(_IRION_BASIC)
		IRIONIMP	udword			CPUFreq;
		IRIONIMP	udword			CPUFreqOk;
#else
		IRIONEXP	udword			CPUFreq=0;
		IRIONEXP	udword			CPUFreqOk=0;
#endif																								// defined(_IRION_BASIC) && !defined(_IRIONBASICS_CPP)
		IRIONAPI	udword			GetCPUFreq	();
		IRIONAPI	inline void		ProfileStart(udword* val);
		IRIONAPI	inline void		ProfileEnd	(udword* val);
		IRIONAPI	inline float	ComputeVBLNb(udword cyclesnb, udword VideoFreq);

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// String management functions
#if defined(_IRION_BASIC)
		inline sdword	StrLen		(ctxt str);
		inline sdword	StrCmp		(ctxt str1, ctxt str2);
		inline sdword	VSNPrintF	(txt buffer, udword size, txt format, va_list args);
		inline sdword	SScanF		(ctxt buffer, ctxt format, void* a);
#endif																								// defined(_IRION_BASIC)

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Debug/Output functions
		IRIONAPI	inline void		PfxTrace(ctxt lpszFormat, ...);

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// File management functions
		#define FILE_READ						0x1
		#define FILE_WRITE						0x2
		#define	FILE_SHAREREAD					0x4
		#define	FILE_SHAREWRITE					0x8

		#define FILE_ATTRB_READONLY             0x00000001  
		#define FILE_ATTRB_HIDDEN               0x00000002  
		#define FILE_ATTRB_SYSTEM               0x00000004  
		#define FILE_ATTRB_DIRECTORY            0x00000008  
		#define FILE_ATTRB_ARCHIVE              0x00000010  
		#define FILE_ATTRB_NORMAL               0x00000020  

		typedef enum {															// File's open flags
			FILE_CREATE_NEW			= 1,										// Create the file, fails if a same filename already exists
			FILE_CREATE_ALWAYS		= 2,										// Create or open an existing file (with truncation to zero)
			FILE_OPEN_EXISTING		= 3,										// Open an existing file, fails if the file doesn't exist
			FILE_OPEN_ALWAYS		= 4,										// Open an existing file, or create one if it doens't exist
			FILE_OPFORCEDW			= 0x7FFFFFFF,
		} FILE_CREATEATTRB;
		typedef enum {
			FILE_FROM_BEGIN			= 1,
			FILE_FROM_CURRENT		= 2, 
			FILE_FROM_END			= 4,
			FILE_FROM_FORCE			= 0x7FFFFFFF,
		} FILE_SEEK;

		IRIONAPI	File*	CreateTpsFile	();
		IRIONAPI	String	GetTempDir		();

#if defined(_IRION_BASIC)
		inline	udword	CreateFile			(ctxt filepathname, udword fileaccess, FILE_CREATEATTRB fileattrib);
		inline	bool	CloseFile			(udword handle);
		inline	udword	SeekFilePtr			(udword handle, udword length, FILE_SEEK seek);
		inline	bool	WriteFile			(udword handle, const void* buffer, udword length, udword* writtenlength=null);
		inline	bool	ReadFile			(udword handle, void* buffer, udword length, udword* readlength=null);
		inline	udword	GetFilePosition		(udword handle);
		inline	udword	GetFileLength		(udword handle);
		inline	bool	DeleteFile			(ctxt filepathname);
		inline	bool	CopyFile			(ctxt existingfile, ctxt newfile, bool overwrite);
		inline	bool	MoveFile			(ctxt existingfile, ctxt newfile);
		inline	udword	GetFileAttributes	(ctxt filepathname);
#endif																								// defined(_IRION_BASIC)
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Threading functions and structures
		typedef enum {
			THP_NORMAL = 1,								// Normal priority level (default level)
			THP_LOWEST,									// Lowest priority level
			THP_HIGHEST									// Highest priority level
					}THPLEVEL;

#if defined(_IRION_BASIC)
		typedef udword (__stdcall *THREAD_ENTRY)(void* param);
				udword	CreateThread	(THREAD_ENTRY entry, void* param);
				bool	TerminateThread	(udword handle, udword exitcode);
				void	ExitThread		(udword exitcode);
				udword	SuspendThread	(udword handle);
				udword	ResumeThread	(udword handle);
				udword	GetCurrentThread();
				bool	SetThreadPriority(udword handle, THPLEVEL prio);
#endif																								// defined(_IRION_BASIC)

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Settings management functions
		IRIONAPI	udword	GetIrionKey		();
		IRIONAPI	udword	GetMachineKey	();
		IRIONAPI	udword	GetCurUserKey	();
		IRIONAPI	udword	CreateKey		(udword basekey, ctxt keyname);
		IRIONAPI	udword	GetKey			(udword basekey, ctxt keyname);
		IRIONAPI	udword	EnumKey			(udword basekey, udword index);

		IRIONAPI	bool	DeleteKey		(udword basekey, ctxt keyname);
		IRIONAPI	bool	DeleteItem		(udword ownerkey, ctxt itemname);

		IRIONAPI	bool	SetItemString	(udword ownerkey, ctxt itemname, ctxt string);
		IRIONAPI	bool	SetItemDWord	(udword ownerkey, ctxt itemname, udword val);
		IRIONAPI	bool	SetItemBinary	(udword ownerkey, ctxt itemname, ubyte* buffer, udword size);

		IRIONAPI	bool	GetItemString	(udword ownerkey, ctxt itemname, String& str);
		IRIONAPI	bool	GetItemDWord	(udword ownerkey, ctxt itemname, udword* val);
		IRIONAPI	bool	GetItemBinary	(udword ownerkey, ctxt itemname, ubyte* buffer, udword* size);

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Library management functions
		IRIONAPI	udword	LoadLibrary		(ctxt libfilename);
		IRIONAPI	bool	FreeLibrary		(udword libhdl);
		IRIONAPI	PROC	GetProcAddress	(udword libhdl, ctxt procname);

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Sorting management functions
		IRIONAPI	bool	ByteSort32		(udword count, udword* source, udword* dest);
	}



inline void	IrionBasics::PfxTrace(ctxt lpszFormat, ...)
{
	va_list args;
	va_start(args, lpszFormat);

	sdword nBuf;
	ubyte szBuffer[512];

	nBuf = _vsnprintf((LPSTR)szBuffer, lstrlen((LPCSTR)szBuffer), (LPCSTR)lpszFormat, args);

	_RPT0(_CRT_WARN, szBuffer);

	va_end(args);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// String Management functions (Accessible by Implementer only)

#if defined(_IRION_BASIC)
	inline sdword IrionBasics::StrLen(ctxt str)
	{
		return strlen((LPCSTR)str);
	}

	inline sdword IrionBasics::StrCmp(ctxt str1, ctxt str2)
	{
		return strcmp ((LPCSTR)str1, (LPCSTR)str2);
	}

	inline sdword IrionBasics::VSNPrintF(txt buffer, udword size, txt format, va_list args)
	{
		return _vsnprintf((LPSTR)buffer, size, (LPCSTR)format, args);
	}

	inline sdword IrionBasics::SScanF(ctxt buffer, ctxt format, void* a)
	{
		return sscanf((const char*)buffer, (const char*)format, a);
	}


#endif																								//defined(_IRION_BASIC)

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// File management functions
#if defined(_IRION_BASIC)

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Create File function
// For fileaccess and fileattrib signification, see the enum definition
// Return -1 if the function fails
inline udword IrionBasics::CreateFile(ctxt filepathname, udword fileaccess, FILE_CREATEATTRB fileattrib)
{
	udword W32FA = ((fileaccess&FILE_READ) ? GENERIC_READ : 0) | ((fileaccess&FILE_WRITE) ? GENERIC_WRITE : 0);
	udword W32FC;
	udword W32S = ((fileaccess&FILE_SHAREREAD)?FILE_SHARE_READ:0) | ((fileaccess&FILE_SHAREWRITE)?FILE_SHARE_WRITE:0);
	switch (fileattrib) {
		case FILE_CREATE_NEW:		W32FC = CREATE_NEW;			break;
		case FILE_CREATE_ALWAYS:	W32FC = CREATE_ALWAYS;		break;
		case FILE_OPEN_EXISTING:	W32FC = OPEN_EXISTING;		break;
		case FILE_OPEN_ALWAYS:		W32FC = OPEN_ALWAYS;		break;
	}
		
	return (udword)::CreateFile((LPCSTR)filepathname, W32FA, W32S, NULL, W32FC, FILE_ATTRIBUTE_NORMAL, NULL);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Close a file
// Return false if the function fails
inline bool IrionBasics::CloseFile(udword handle)
{
	return CloseHandle((HANDLE)handle)!=0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Seek the file pointer
// Return -1 if the function fails
inline udword IrionBasics::SeekFilePtr(udword handle, udword length, FILE_SEEK seek)
{
	udword W32S = ((seek&FILE_FROM_BEGIN) ? FILE_BEGIN : 0) | ((seek&FILE_FROM_CURRENT) ? FILE_CURRENT : 0) | ((seek&FILE_FROM_END) ? FILE_END : 0);
	return SetFilePointer((HANDLE)handle, length, NULL, W32S);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Return false if the function fails
inline bool IrionBasics::WriteFile(udword handle, const void* buffer, udword length, udword* writtenlength)
{
	udword written;
	return ::WriteFile((HANDLE)handle, buffer, length, (DWORD*)(writtenlength?writtenlength:&written), NULL)!=0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Return false if the function fails
inline bool	IrionBasics::ReadFile(udword handle, void* buffer, udword length, udword* readlength)
{
	udword read;
	return ::ReadFile((HANDLE)handle, buffer, length, (DWORD*)(readlength?readlength:&read), NULL)!=0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Return -1 if the function fails
inline udword IrionBasics::GetFilePosition(udword handle)
{
	return SetFilePointer((HANDLE)handle, 0, NULL, FILE_CURRENT);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Return -1 if the function fails
inline udword IrionBasics::GetFileLength(udword handle)
{
	return GetFileSize((HANDLE)handle, NULL);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Returns true if successful
inline bool IrionBasics::DeleteFile(ctxt filepathname)
{
	return ::DeleteFile((LPCSTR)filepathname)!=0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Returns true if successful
inline bool	IrionBasics::CopyFile(ctxt existingfile, ctxt newfile, bool overwrite)
{
	return ::CopyFile((LPCSTR)existingfile, (LPCSTR)newfile, !overwrite)!=0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Returns true if successful
inline bool	IrionBasics::MoveFile(ctxt existingfile, ctxt newfile)
{
	return ::MoveFile((LPCSTR)existingfile, (LPCSTR)newfile)!=0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Returns -1 if the function fails
inline udword IrionBasics::GetFileAttributes(ctxt filepathname)
{
	udword W32ATR = ::GetFileAttributes((LPCSTR)filepathname);
	if (W32ATR==-1)													return -1;

	udword ATR=0;
	if (W32ATR & FILE_ATTRIBUTE_READONLY	)		ATR	|=	FILE_ATTRB_READONLY;
	if (W32ATR & FILE_ATTRIBUTE_HIDDEN		)		ATR	|=	FILE_ATTRB_HIDDEN;
	if (W32ATR & FILE_ATTRIBUTE_SYSTEM		)		ATR	|=	FILE_ATTRB_SYSTEM;
	if (W32ATR & FILE_ATTRIBUTE_DIRECTORY	)		ATR	|=	FILE_ATTRB_DIRECTORY;
	if (W32ATR & FILE_ATTRIBUTE_ARCHIVE		)		ATR	|=	FILE_ATTRB_ARCHIVE;
	if (W32ATR & FILE_ATTRIBUTE_NORMAL		)		ATR	|=	FILE_ATTRB_NORMAL;
	return	ATR;
}

#endif																								//defined(_IRION_BASIC)

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Debug and output functions
#ifdef _DEBUG
	#define ASSERT(exp)				(void)( (exp) || (_assert(#exp, __FILE__, __LINE__), 0) )
	#define TRACE(x)				IrionBasics::PfxTrace(ctxt("%s"), x)
	#define TRACE1(x, a)			IrionBasics::PfxTrace((ctxt)x, a)
	#define TRACE2(x, a, b)			IrionBasics::PfxTrace((ctxt)x, a, b)
	#define TRACE3(x, a, b, c)		IrionBasics::PfxTrace((ctxt)x, a, b, c)
	#define TRACE4(x, a, b, c, d)	IrionBasics::PfxTrace((ctxt)x, a, b, c, d)
#else
	#define	ASSERT(x)				((void)0)
	#define TRACE(x)				((void)0)
	#define TRACE1(x, a)			((void)0)
	#define TRACE2(x, a, b)			((void)0)
	#define TRACE3(x, a, b, c)		((void)0)
	#define TRACE4(x, a, b, c, d)	((void)0)
#endif																								//_DEBUG


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Profiling functions implementation
inline void	IrionBasics::ProfileStart(udword* val)
{
	__asm {
		push	edx
		push	eax
		rdtsc
		mov		ebx, val
		mov		[ebx], eax
		pop		eax
		pop		edx
	}
}

inline void	IrionBasics::ProfileEnd(udword* val)
{
	__asm {
		push	edx
		push	eax
		rdtsc
		mov		ebx, val
		sub		eax, [ebx]
		mov		[ebx], eax
		pop		eax
		pop		edx
	}
}

inline float IrionBasics::ComputeVBLNb(udword cyclesnb, udword VideoFreq)
{
	return (float)cyclesnb/(float)(GetCPUFreq()/VideoFreq);
}






*/