///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains a custom array class.
 *	\file		IceCustomArray.h
 *	\author		Pierre Terdiman
 *	\date		January, 15, 1999
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICECUSTOMARRAY_H__
#define __ICECUSTOMARRAY_H__

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// CONSTANTS
	#define CUSTOMARRAY_BLOCKSIZE	(4*1024)		// 4 Kb => heap size

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//
	//																					CustomArray Class Definition
	//
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	class ICECORE_API CustomArray
	{
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Structures and enums
		struct CustomBlock{
			CustomBlock()		{ Addy = null; }
			~CustomBlock()		{ DELETEARRAY(Addy); }
			void*				Addy;						// Stored data
			udword				Size;						// Length of stored data
			udword				Max;						// Heap size
		};

		struct CustomCell{
			CustomCell()		{ NextCustomCell = null; }

			struct CustomCell*	NextCustomCell;
			CustomBlock			Item;
		};

	public:
		// Constructor / Destructor
		CustomArray(udword startsize=CUSTOMARRAY_BLOCKSIZE, void* inputbuffer=null);
		CustomArray(const char* filename);
		~CustomArray();

		CustomArray&			Clean();

		// Store methods
		CustomArray&			StoreBit(bool bit);
		CustomArray&			EndBits();

		CustomArray&			Store(BOOL Bo);
		CustomArray&			Store(bool Bo);
		CustomArray&			Store(char b);
		CustomArray&			Store(ubyte b);
		CustomArray&			Store(short w);
		CustomArray&			Store(uword w);
		CustomArray&			Store(long d);
		CustomArray&			Store(unsigned long d);
	//	CustomArray&			Store(int d);
		CustomArray&			Store(unsigned int d);
		CustomArray&			Store(float f);
		CustomArray&			Store(double f);
		CustomArray&			Store(const char* String);
		CustomArray&			Store(void* buffer, udword size);

		CustomArray&			StoreASCII(BOOL Bo);
		CustomArray&			StoreASCII(bool Bo);
		CustomArray&			StoreASCII(char b);
		CustomArray&			StoreASCII(ubyte b);
		CustomArray&			StoreASCII(short w);
		CustomArray&			StoreASCII(uword w);
		CustomArray&			StoreASCII(long d);
		CustomArray&			StoreASCII(unsigned long d);
	//	CustomArray&			StoreASCII(int d);
		CustomArray&			StoreASCII(unsigned int d);
		CustomArray&			StoreASCII(float f);
		CustomArray&			StoreASCII(double f);
		CustomArray&			StoreASCII(const char* String);

		// Management methods
		bool					ExportToDisk(const char* FileName);
		bool					ExportToDisk(FILE* fp);

		udword					GetOffset();
		CustomArray&			Padd();
		CustomArray&			LinkTo(CustomArray* array);
		void*					GetAddress();
		udword					GetCellUsedSize();
		void*					Collapse(void* userbuffer=null);

		// Address methods
		bool					PushAddress();
		CustomArray&			PopAddressAndStore(BOOL Bo);
		CustomArray&			PopAddressAndStore(bool Bo);
		CustomArray&			PopAddressAndStore(char b);
		CustomArray&			PopAddressAndStore(ubyte b);
		CustomArray&			PopAddressAndStore(short w);
		CustomArray&			PopAddressAndStore(uword w);
		CustomArray&			PopAddressAndStore(long d);
		CustomArray&			PopAddressAndStore(unsigned long d);
	//	CustomArray&			PopAddressAndStore(int d);
		CustomArray&			PopAddressAndStore(unsigned int d);
		CustomArray&			PopAddressAndStore(float f);
		CustomArray&			PopAddressAndStore(double f);

		// Read methods
		ubyte					GetByte();					//!< Read a byte from the array
		uword					GetWord();					//!< Read a word from the array
		udword					GetDword();					//!< Read a dword from the array
		float					GetFloat();					//!< Read a float from the array
		ubyte*					GetString();				//!< Read a string from the array

		CustomArray&			Reset(udword offset=0);
		void*					GetChunk(const char* chunk);

	private:
		CustomCell*				mCurrentCell;				// Current block cell
		CustomCell*				mInitCell;					// First block cell

		void*					mCollapsed;					// Possible collapsed buffer
		void**					mAddresses;					// Stack to store addresses
		void*					mLastAddress;				// Last address used in current block cell
		uword					mNbPushedAddies;			// #saved addies
		uword					mNbAllocatedAddies;			// #allocated addies
		ubyte					mBitCount;
		ubyte					mBitMask;

		// Management methods
		CustomArray&			Empty();
		CustomArray&			CheckArray(udword bytesneeded);
		CustomArray&			NewBlock(CustomCell* previouscell, udword size=0);
		bool					SaveCell(CustomCell* p, FILE* fp);
		CustomArray&			StoreASCIICode(char code);
	};

#endif // __ICECUSTOMARRAY_H__
