///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains a dynamic list class.
 *	\file		IceDynamicList.h
 *	\author		Pierre Terdiman
 *	\date		December, 12, 1999
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEDYNAMICLIST_H__
#define __ICEDYNAMICLIST_H__

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// CONSTANTS
	#define DEFAULT_DYNAMICLIST_SIZE		2
	#define DEFAULT_FREEINDICESLIST_SIZE	2


	class ICECORE_API DynamicList
	{
	public:
		// Constructor / Destructor
							DynamicList();
							~DynamicList();

		// Basic usage
		uword				Add(udword value);
		void				Remove(uword virtualindex);

		// Physical data access
		udword				GetMaxNbEntries()		{ return mMaxNbEntries;		}	//!< Returns max number of entries		
		udword				GetNbEntries()			{ return mCurrentNbEntries;	}	//!< Returns current number of entries		
		udword*				GetList()				{ return mList;				}	//!< Gets the complete list
		udword				GetValue(uword virtualindex);							// Returns real value according to virtual index.

	protected:
		// Physical list
		udword*				mList;													//!< Physical list, with no holes but unsorted.
		uword				mCurrentNbEntries;										//!< Current number of values in the physical list.
		uword				mMaxNbEntries;											//!< Maximum possible number of values in the physical list.

		// Cross-references
		uword*				mOutToIn;												//!< Maps virtual indices to real ones.
		uword*				mInToOut;												//!< Maps real indices to virtual ones.

		// Recycled locations
		uword*				mFreeIndices;											//!< The recycle list = a list of current free indices.
		uword				mNbFreeIndices;											//!< Current number of free indices in the recycle list.
		uword				mMaxNbFreeIndices;										//!< Maximum possible number of free indices in the recycle list.
	};

#endif //__ICEDYNAMICLIST_H__


