///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains import-export related code.
 *	\file		IceImportExport.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEIMPORTEXPORT_H__
#define __ICEIMPORTEXPORT_H__

	//! TO BE DOCUMENTED
	#define OFFSET_OF(Class, Member)	(size_t)&(((Class*)0)->Member)
	//! TO BE DOCUMENTED
	#define ARRAYSIZE(p)				(sizeof(p)/sizeof(p[0]))

	//! TO BE DOCUMENTED
	enum eFieldType
	{
		FIELD_BOOL				= 0,			// a bool (not a BOOL!)
		FIELD_BYTE				= 1,			// sbyte or ubyte
		FIELD_WORD				= 2,			// sword or uword
		FIELD_DWORD				= 3,			// sdword or udword
		FIELD_FLOAT				= 4,			// float
		FIELD_DOUBLE			= 5,			// double
		FIELD_CELLPTR			= 6,			// Cell pointer (or any pointer to a derived class)
		FIELD_FORCE_DWORD		= 0x7fffffff
	};

	// FieldType is used as an offset in the following array
	//! TO BE DOCUMENTED
	ubyte gFieldSize[] = {
		sizeof(bool),
		sizeof(ubyte),
		sizeof(uword),
		sizeof(udword),
		sizeof(float),
		sizeof(double),
		sizeof(void*),
	};

	// Flags
	#define F_CANEDIT		(1<<0)				// Editable field

	//! TO BE DOCUMENTED
	#define _FIELD(type, name, fieldtype, count, flags)				{ fieldtype, #name, OFFSET_OF(type, name), count, flags }

	//! TO BE DOCUMENTED
	#define DEFINE_FIELD(type, name, fieldtype, flags)				_FIELD(type, name, fieldtype,     1, flags)
	//! TO BE DOCUMENTED
	#define DEFINE_ARRAY(type, name, fieldtype, count, flags)		_FIELD(type, name, fieldtype, count, flags)

	//! TO BE DOCUMENTED
	struct FieldDescriptor
	{
		eFieldType		Type;
		char*			Name;
		udword			Offset;
		uword			Count;
		uword			Flags;
	};




	//! TO BE DOCUMENTED
	#define CLASS_DESCRIPTOR																			\
																										\
	static	FieldDescriptor		mClassDescriptor[];														\
																										\
	/*virtual*/ udword GetDescriptorSize();																\
	FieldDescriptor* GetDescriptor() { return mClassDescriptor; }										\

	//! TO BE DOCUMENTED
	#define DECLARE_IMPORT_EXPORT(currentclass, baseclass)												\
																										\
	CLASS_DESCRIPTOR;																					\
																										\
	virtual bool GetEditableFields(Container& edit)														\
	{																									\
		baseclass::GetEditableFields(edit);																\
		FieldDescriptor* Fields = currentclass::GetDescriptor();										\
		for(udword i=0;i<currentclass::GetDescriptorSize();i++)											\
		{																								\
			if(Fields[i].Flags&F_CANEDIT)	edit.Add((udword)&Fields[i]);								\
		}																								\
		return true;																					\
	}																									\
																										\
	virtual udword Export(ExportContext& ec)															\
	{																									\
		if(!baseclass::Export(ec))	return 0;															\
		else return ec.Save(#currentclass, this, mClassDescriptor, currentclass::GetDescriptorSize());	\
	}																									\
																										\
	virtual udword Import(ImportContext& ic)															\
	{																									\
		if(!baseclass::Import(ic))	return 0;															\
		else return ic.Load(#currentclass, this, mClassDescriptor, currentclass::GetDescriptorSize());	\
	}


	class Cell;

	class ICECORE_API ImportContext
	{
		public:
					ImportContext();
					~ImportContext();
		udword		Load(const char* classname, Cell* currentclass, FieldDescriptor* fields, udword nbfields);
	};

	class ICECORE_API ExportContext
	{
		public:
					ExportContext();
					~ExportContext();
		udword		Save(const char* classname, Cell* currentclass, FieldDescriptor* fields, udword nbfields);
	};

#endif // __ICEIMPORTEXPORT_H__
