///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains code for ICE kernel.
 *	\file		IceKernel.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEKERNEL_H__
#define __ICEKERNEL_H__

	ICECORE_API VTABLE GetVTable(Cell* cell);

	//! Reverse all the bits in a 32 bit word (from Steve Baker's Cute Code Collection)
	__forceinline void ReverseBits(udword& n)
	{
		n = ((n >>  1) & 0x55555555) | ((n <<  1) & 0xaaaaaaaa) ;
		n = ((n >>  2) & 0x33333333) | ((n <<  2) & 0xcccccccc) ;
		n = ((n >>  4) & 0x0f0f0f0f) | ((n <<  4) & 0xf0f0f0f0) ;
		n = ((n >>  8) & 0x00ff00ff) | ((n <<  8) & 0xff00ff00) ;
		n = ((n >> 16) & 0x0000ffff) | ((n << 16) & 0xffff0000) ;
	}

	//! Count the number of '1' bits in a 32 bit word (from Steve Baker's Cute Code Collection)
	__forceinline void CountBits(udword& n)
	{
		n = (n & 0x55555555) + ((n & 0xaaaaaaaa) >> 1);
		n = (n & 0x33333333) + ((n & 0xcccccccc) >> 2);
		n = (n & 0x0f0f0f0f) + ((n & 0xf0f0f0f0) >> 4);
		n = (n & 0x00ff00ff) + ((n & 0xff00ff00) >> 8);
		n = (n & 0x0000ffff) + ((n & 0xffff0000) >> 16);
	}

	//! Test to see if a number is an exact power of two (from Steve Baker's Cute Code Collection)
	__forceinline bool IsPowerOfTwo(udword n)				{ return ((n&(n-1))==0);			}

	//! Zero the least significant '1' bit in a word. (from Steve Baker's Cute Code Collection)
	__forceinline void ZeroLeastSetBit(udword& n)			{ n&=(n-1);							}

	//! Set the least significant N bits in a word. (from Steve Baker's Cute Code Collection)
	__forceinline void SetLeastNBits(udword& x, udword n)	{ x|=~(~0<<n);						}

	//! Classic XOR swap (from Steve Baker's Cute Code Collection)
	__forceinline void Swap(udword& x, udword& y)			{ x ^= y; y ^= x; x ^= y;			}

	//! Little/Big endian (from Steve Baker's Cute Code Collection)
	__forceinline char LittleEndian()						{ int i = 1; return *((char*)&i);	}

	// Returns a unit random floating-point value
	__forceinline float UnitRandomFloat()					{ return float(rand()) * ONE_OVER_RAND_MAX;	}

	class ICECORE_API Kernel
	{
	public:
		// Constructor/Destructor
			Kernel();
			~Kernel();

			bool				Register(Cell* object);
			bool				Unregister(Cell* object);

			bool				CreateRef(Cell* owner, Cell* ref);
			bool				DeleteRef(Cell* owner, Cell* ref);

			udword				GetNbRefs(Cell* cell);
			udword				GetNbOwners(Cell* cell);
			Cell*				GetOwner(Cell* cell, udword i);
			Cell*				GetRef(Cell* cell, udword i);

			//
			DynamicList*		GetTranslator()		{ return &mTranslator;	}
			RefTracker*			GetRefTracker()		{ return &mTracker;		}
	private:
			RefTracker			mTracker;
			DynamicList			mTranslator;
	};

#endif // __ICEKERNEL_H__
