///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains profiling code.
 *	\file		IceProfiler.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEPROFILER_H__
#define __ICEPROFILER_H__

	//!	This function starts recording the number of cycles elapsed.
	//!	\param	val		address of a 32 bits value where the system should store the result.
	//!	\see	EndProfile
	__forceinline void	StartProfile(udword* val)
	{
		__asm {
			rdtsc
			mov		ebx, val
			mov		[ebx], eax
		}
	}

	//!	This function ends recording the number of cycles elapsed.
	//!	\param		address to store the number of cycles elapsed since the last StartProfile.
	//!	\see		StartProfile
	__forceinline void	EndProfile(udword* val)
	{
		__asm {
			rdtsc
			mov		ebx, val
			sub		eax, [ebx]
			mov		[ebx], eax
		}
	}

#endif // __ICEPROFILER_H__
