///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains the reference tracking system.
 *	\file		IceRefTracker.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEREFTRACKER_H__
#define __ICEREFTRACKER_H__

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// CONSTANTS
	#define	REFTRACK_INVALIDOFFSETS		(1<<0)

	class ICECORE_API RefTracker
	{
	public:
		// Constructor/Destructor
			RefTracker();
			~RefTracker();

			bool			AddRef(udword owner, udword ref);
			bool			RemoveRef(udword owner, udword ref);
			bool			RemoveIndex(udword id);

			udword			GetNbMarkedIndices()				{	return mMarkedIndices.GetNbEntries();	}
			udword*			GetMarkedIndices()					{	return mMarkedIndices.GetEntries();		}

			udword			GetNbMarkedLinks()					{	return mMarkedLinks.GetNbEntries();		}
			udword*			GetMarkedLinks()					{	return mMarkedLinks.GetEntries();		}

			udword			GetNbRefs(udword owner);
			udword			GetNbOwners(udword ref);

			udword			GetOwner(udword ref, udword i);
			udword			GetRef(udword owner, udword i);

			udword			GetUsedRam();

	private:
			udword			mFlags;

			udword			mMaxNbPairs;
			udword			mCurNbPairs;
			udword*			mOwners;
			udword*			mRefs;

			udword*			mSortedOwners;
			udword*			mSortedRefs;

			udword*			mOwnersOffsets;
			udword*			mRefsOffsets;

			udword			mMinOwner;
			udword			mMinRef;
			udword			mMaxOwner;
			udword			mMaxRef;

			Container		mMarkedLinks;
			Container		mMarkedIndices;

			// Methods
			bool			CreateOffsets();
			bool			PurgeList();
			udword			GetPairIndex(udword owner, udword ref);
			bool			MarkLink(udword owner, udword ref);
			bool			MarkIndex(udword id);
	};

#endif // __ICEREFTRACKER_H__
