///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains all string-related code.
 *	\file		IceString.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICESTRING_H__
#define __ICESTRING_H__

	class ICECORE_API String
	{
	public:
			// Constructors/Destructor
						String();
						String(const char* string);
						String(String& str);
						~String();

			bool		Set(const char* string);					// Assign a string
			udword		Length()					const;			// Get the string's length
			udword		Compare(const String& str2)	const;			// Compare the String with another one
			String&		ToUpper();									// Transform into uppercase
			String&		ToLower();									// Transform into lowercase
			bool		GetParameter(String& param);				// Parameters parsing

			// Cast operators
			//! Cast operator for const char* = String
		__forceinline	operator const char*()		const			{ return (const char*)(mText);	}
			//! Cast operator for char* = String
		__forceinline	operator char*()			const			{ return (char*)(mText);		}

			// Assignment operators
			//! Operator for String = const char*
				String&	operator=(const char* string)				{ Set(string);				return *this; }
			//! Operator for String = char*
				String&	operator=(char* string)						{ Set((const char*)string);	return *this; }
			//! Operator for String = String
				String&	operator=(String& string)					{ Set(string);				return *this; }

			//! Concatenation operator for String += String
				String&	operator+=(const String& string);

			char*		mText;		//!< The character string held in the class

	private:
			char*		NextParameter(char* string, String& param);
	};

	ICECORE_API	inline bool operator==(const String& s1, const String& s2)	{ return s1.Compare(s2) == 0; }	//!< Operator for String == String
	ICECORE_API	inline bool operator!=(const String& s1, const String& s2)	{ return s1.Compare(s2) != 0; }	//!< Operator for String != String
	ICECORE_API	inline bool operator< (const String& s1, const String& s2)	{ return s1.Compare(s2) <  0; }	//!< Operator for String < String
	ICECORE_API	inline bool operator> (const String& s1, const String& s2)	{ return s1.Compare(s2) >  0; }	//!< Operator for String > String
	ICECORE_API	inline bool operator<=(const String& s1, const String& s2)	{ return s1.Compare(s2) <= 0; }	//!< Operator for String <= String
	ICECORE_API	inline bool operator>=(const String& s1, const String& s2)	{ return s1.Compare(s2) >= 0; }	//!< Operator for String >= String

#endif // __ICESTRING_H__
