///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	This file contains the controller-related exported structures.
 *	\file		IceAddController.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEADDCONTROLLER_H__
#define __ICEADDCONTROLLER_H__

	enum ControllerType{
		CTRL_FLOAT			= 1,			// A single float value
		CTRL_VECTOR			= 2,			// A point/vector (x,y,z, 3 floats)
		CTRL_QUAT			= 3,			// A quaternion (x,y,z,w, 4 floats)
		CTRL_PR				= 4,			// A PR structure (Position, Rotation = vector, quat = 7 floats)
		CTRL_PRS			= 5,			// A PRS structure (Position, Rotation, Scale = vector, quat, vector = 10 floats)
		CTRL_FORCE_DWORD	= 0x7fffffff
	};

	// Handy helper structures.
	struct CtrlFloat	{ float Value;									};
	struct CtrlVector	{ float x, y, z;								};
	struct CtrlQuat		{ float x, y, z, w;								};
	struct CtrlPR		{ float tx, ty, tz, qx, qy, qz, qw;				};
	struct CtrlPRS		{ float tx, ty, tz, qx, qy, qz, qw, sx, sy, sz;	};

	enum ControllerMode{
		CTRL_SAMPLES		= 1,			// Sampling mode
		CTRL_KEYFRAMES		= 2,			// Keyframing mode
		CTRL_DWORD			= 0x7fffffff
	};

	struct ControllerDataCreate
	{
		ControllerType		Type;
		ControllerMode		Mode;
	};

	// Base class
	class ControllerData
	{
	public:
										ControllerData();
		virtual							~ControllerData();

		virtual	bool					Init(ControllerDataCreate* create)	= 0;

	public:
				ControllerType			mType;					//!< Type of data handled by the controller
				ControllerMode			mMode;					//!< Data are saved as samples, keyframes, etc
				udword					mFlags;					//!< Possible flags [not used yet]
	};

	struct SampleDataCreate : ControllerDataCreate
	{
		udword				NbSamples;
		udword				SamplingRate;
	};

	// A sample container.
	class SampleData : public ControllerData
	{
	public:
										SampleData();
		virtual							~SampleData();

		virtual	bool					Init(ControllerDataCreate* create);

	public:
				udword					mNbSamples;				//!< Number of samples
				udword					mSamplingRate;			//!< Sampling rate
				void*					mSamples;				//!< List of mNbSamples samples, whose type is mType
	};

	//! The controller information class.
	class ControllerInfo
	{
	public:
										ControllerInfo();
		virtual							~ControllerInfo();

			// Database management
				String					mField;					//!< Controlled field
				sdword					mObjectID;				//!< Controller's ID
				sdword					mOwnerID;				//!< Owner object's ID
			// NB: that way I added the animation controllers without breaking the old format.

			// Animation data
				ControllerData*			mData;					//!< Actual animation data
	};

#endif // __ICEADDCONTROLLER_H__
