///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	This file contains the base class for all export formats.
 *	\file		IceBaseFormat.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEBASEFORMAT_H__
#define __ICEBASEFORMAT_H__

	class ExportFormat
	{
		public:
	// Constructor/Destructor
										ExportFormat()	{}
		virtual							~ExportFormat()	{}

	// Exporter init
		virtual	bool					Init(bool motion)								= 0;

	// Main scene info
		virtual bool					SetSceneInfo(MainInfo& maininfo)				= 0;

	// Export loop
		virtual bool					ExportCamera		(CameraInfo& camera)		= 0;
		virtual bool					ExportController	(ControllerInfo& controller)= 0;
		virtual bool					ExportHelper		(HelperInfo& helper)		= 0;
		virtual bool					ExportLight			(LightInfo& light)			= 0;
		virtual bool					ExportMaterial		(MaterialInfo& material)	= 0;
		virtual bool					ExportMesh			(MeshInfo& mesh)			= 0;
		virtual bool					ExportMotion		(MotionInfo& motion)		= 0;
		virtual bool					ExportShape			(ShapeInfo& shape)			= 0;
		virtual bool					ExportTexture		(TextureInfo& texture)		= 0;

	// End of export notification
		virtual bool					EndExport(StatsInfo& stats)						= 0;

	// Exportation settings
				ExportFormat&			SetSettings(ExportSettings& settings)	{ mSettings = settings; return *this; }
				ExportFormat&			GetSettings(ExportSettings& settings)	{ settings = mSettings; return *this; }

	// Output filename
				ExportFormat&			SetOutputFilename(String& str)			{ mFilename = str;		return *this; }

		public:
				ExportSettings			mSettings;		//!< Exportation settings
				String					mFilename;		//!< Output filename
	};

#endif // __ICEBASEFORMAT_H__
