///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	This file contains all the basic information structures.
 *	\file		IceBaseInfo.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEBASEINFO_H__
#define __ICEBASEINFO_H__

	//! This class holds the main information about the scene
	class MainInfo
	{
	public:
				//! Constructor
				MainInfo()
				{
					NbNodes				= 0;
					NbCharacters		= 0;
				}
				TimeInfo				Time;					//!< Time-related information
				udword					NbNodes;				//!< Number of expected nodes
				udword					NbCharacters;			//!< Number of expected characters
				MAXPoint				BackColor;				//!< Background color [animatable]
				MAXPoint				AmbColor;				//!< Ambient light color [animatable]
	};

	//! The base information class for each object
	class ObjectInfo
	{
	public:
				//! Constructor
				ObjectInfo()
				{
					mName				= null;
					mIsGroupMember		= false;
					mLocalPRS			= false;
					mObjectID			= -1;
					mParentID			= -1;
					mLinkID				= -1;
					mWireColor			= 0;
					mUserProps			= null;
				}
				//! Destructor
		virtual	~ObjectInfo()			{}

			// Database management
				ubyte*					mName;					//!< Object's name
				bool					mIsGroupMember;			//!< Belonging to a group
				sdword					mObjectID;				//!< Object's ID
				sdword					mParentID;				//!< Father's ID
				sdword					mLinkID;				//!< Possible link reference
			// Common properties
				MAXPRS					mPrs;					//!< Position/Rotation/Scale
				bool					mLocalPRS;				//!< True for local PRS, false for absolute ones.
				bool					mD3DCompliant;			//!< Object has been converted to D3D frame
				udword					mWireColor;				//!< Object's Wire Color in MAX
				ubyte*					mUserProps;				//!< User-defined properties
	};

	//! Statistics class
	class StatsInfo
	{
	public:
				//! Constructor
				StatsInfo(){
					mNbGeomObjects		= 0;
					mNbDerivedObjects	= 0;
//					mNbBPatchObjects	= 0;
					mNbCameras			= 0;
					mNbLights			= 0;
					mNbShapes			= 0;
					mNbHelpers			= 0;
					mNbControllers		= 0;
					mNbMaterials		= 0;
					mNbTexmaps			= 0;
					mNbUnknowns			= 0;
					mNbInvalidNodes		= 0;
//					mNbBipedParts		= 0;
//					mNbCharacters		= 0;
				}
				//! Destructor
				~StatsInfo(){}

				udword					mNbGeomObjects;			//!< Number of geomobjects found
				udword					mNbDerivedObjects;		//!< Number of derived objects found (actually number of skins)
//				udword					mNbBPatchObjects;		//!< Number of BPatch objects found
				udword					mNbCameras;				//!< Number of cameras found
				udword					mNbLights;				//!< Number of lights found
				udword					mNbShapes;				//!< Number of shapes found
				udword					mNbHelpers;				//!< Number of helpers found
				udword					mNbControllers;			//!< Number of controllers found
				udword					mNbMaterials;			//!< Number of materials found
				udword					mNbTexmaps;				//!< Number of texmaps found
				udword					mNbUnknowns;			//!< Number of unknown nodes found
				udword					mNbInvalidNodes;		//!< Number of invalid nodes found
//				udword					mNbBipedParts;			//!< Number of biped nodes found
//				udword					mNbCharacters;			//!< Number of characters found
	};

#endif // __ICEBASEINFO_H__
