///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	This file contains the exportation settings structure.
 *	\file		IceExportSettings.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEEXPORTSETTINGS_H__
#define __ICEEXPORTSETTINGS_H__

	//! Exportation settings. You can here customize the way Flexporter exports information from MAX.
	class ExportSettings
	{
	public:
					//! Constructor
					ExportSettings()
					{
						Compressed					= false;
						ExportWholeScene			= true;
						ExportVisTrack				= false;
						ExportHiddenNodes			= false;

						ExpUVW						= false;
						DiscardW					= true;
						ExpVTXColor					= false;
						UseSmgrp					= true;
						RemoveScaling				= true;
						ConvertToD3D				= true;
						AbsolutePRS					= false;
						ConvexHull					= false;
						BoundingSphere				= false;
						InertiaTensor				= false;

						ConsolidateMesh				= false;
						ComputeFaceNormals			= false;
						ComputeVertexNormals		= false;
						ExportNormalInfo			= false;

						ForceAmbient				= false;
						ForceDiffuse				= false;

						OpacityInAlpha				= true;
						TexnameOnly					= false;
						TexMaxSize					= 256;
						KeepAmbientTexture			= false;
						KeepDiffuseTexture			= true;
						KeepSpecularTexture			= false;
						KeepShininessTexture		= false;
						KeepShiningStrengthTexture	= false;
						KeepSelfIllumTexture		= false;
						KeepOpacityTexture			= true;
						KeepFilterTexture			= false;
						KeepBumpTexture				= false;
						KeepReflexionTexture		= false;
						KeepRefractionTexture		= false;
						KeepDisplacementTexture		= false;

						ExpCameras					= true;
						ExpTargetNodes				= false;

						ExpLights					= true;

						SingleFrame					= true;
						Sampling					= true;
						SamplingRate				= 1;
					}

					// General settings
		bool		Compressed;					//!< Compress output file or not [not recoded yet]
		bool		ExportWholeScene;			//!< Export whole scene or single node
		bool		ExportVisTrack;				//!< Export visibility track
		bool		ExportHiddenNodes;			//!< Export hidden nodes or discard them

					// Meshes settings
//		bool		CreateStrips;				//!< Create triangle strips or not
		bool		ExpUVW;						//!< Export UVW mappings or discard them
		bool		DiscardW;					//!< Discard W in UVW or keep it
		bool		ExpVTXColor;				//!< Export vertex colors or discard them
		bool		UseSmgrp;					//!< Use smoothing groups or discard them
//		bool		ExpNorm;					//!< Export normals or not
//		bool		LOD;						//!< Compute progressive meshes or not
		bool		RemoveScaling;				//!< Remove non-uniform scaling of all nodes
		bool		ConvertToD3D;				//!< Convert from MAX frame to D3D frame
		bool		AbsolutePRS;				//!< Export absolute PRS values (else relative to parent)
		bool		ConvexHull;					//!< Flexporter can compute the convex hull for you
		bool		BoundingSphere;				//!< Flexporter can compute the bounding sphere for you
		bool		InertiaTensor;				//!< Flexporter can compute the inertia tensor for you

					// Consolidation settings
		bool		ConsolidateMesh;			//!< Flexporter can rebuild a clean mesh for you
		bool		ComputeFaceNormals;			//!< Flexporter can compute and export face normals
		bool		ComputeVertexNormals;		//!< Flexporter can compute and export vertex normals
		bool		ExportNormalInfo;			//!< Export NormalInfo field or not

					// Material settings
		bool		ForceAmbient;				//!< Force material ambient color to pure white.
		bool		ForceDiffuse;				//!< Force material diffuse color to pure white.

					// Texture settings
		bool		OpacityInAlpha;				//!< Pack the opacity map in the diffuse map alpha channel
		bool		TexnameOnly;				//!< Only reports the texture's name, not the pixel buffer
		udword		TexMaxSize;					//!< Texture maximum size
		bool		KeepAmbientTexture;			//!< Export ambient textures, or discard them
		bool		KeepDiffuseTexture;			//!< Export diffuse textures, or discard them
		bool		KeepSpecularTexture;		//!< Export specular textures, or discard them
		bool		KeepShininessTexture;		//!< Export shininess textures, or discard them
		bool		KeepShiningStrengthTexture;	//!< Export shining strength textures, or discard them
		bool		KeepSelfIllumTexture;		//!< Export self-illum textures, or discard them
		bool		KeepOpacityTexture;			//!< Export opacity textures, or discard them
		bool		KeepFilterTexture;			//!< Export filter textures, or discard them
		bool		KeepBumpTexture;			//!< Export bump textures, or discard them
		bool		KeepReflexionTexture;		//!< Export reflexion textures, or discard them
		bool		KeepRefractionTexture;		//!< Export refraction textures, or discard them
		bool		KeepDisplacementTexture;	//!< Export displacement textures, or discard them

					// Camera settings
		bool		ExpCameras;					//!< Export cameras or discard them
		bool		ExpTargetNodes;				//!< Export target nodes or discard them

					// Light settings
		bool		ExpLights;					//!< Export lights or discard them

					// Animation settings
		bool		SingleFrame;				//!< Export a single frame
		bool		Sampling;					//!< Export samples (true) or keyframes (false)
		long		SamplingRate;				//!< Sampling rate
	};

#endif // __ICEEXPORTSETTINGS_H__
